library reflector.src.interfaces;

/// Generic interface for reflection. Can be implemented a variety of ways:
/// 1. Runtime reflection (using dart:mirrors)
/// 2. Build-time transformer (using package:analyzer)
/// 3. Your own custom solution.
///
/// A reflector can be scoped to a specific library or collection of types, or
/// even be pre-compiled at build-time. See [call] for more details.
abstract class Reflector implements Function {
  /// Returns all metadata that match the properties defined in [query].
  /// The [query] will also define what parts of the metadata are retained.
  ///
  /// If a [query] is not possible (e.g., a limited static analysis version of
  /// the reflector is used, and a query is requested on something that is not
  /// predetermined) then a [StateError] is thrown.
  Iterable<Metadata> call([Query query]);
}

/// Generic interface for metadata. Also see [RuntimeMetadata].
abstract class Metadata {
  /// Annotations on the item.
  final List annotations;

  /// The name of the metadata item. For example: `#Foo`.
  final Symbol simpleName;

  /// The fully qualified name of the metadata item.
  /// For example: `const Symbol('my_library.Foo')`.
  final Symbol qualifiedName;

  const Metadata._(this.annotations, this.simpleName, this.qualifiedName);

  /// The type, whether the field or method type or class type.
  ///
  /// *This differs based on the platform (Runtime or Analysis), because in
  /// analysis mode there is no Dart type available.*
  get type;
}

/// Generic interface for metadata that is available at application runtime.
abstract class RuntimeMetadata extends Metadata {
  /// The runtime type, whether the field or method type or class type.
  final Type type;

  const RuntimeMetadata._(
      this.type,
      List annotations,
      Symbol simpleName,
      Symbol qualifiedName)
          : super._(annotations, simpleName, qualifiedName);
}

/// Generic interface for metadata information about a class [Type].
class ClassRuntimeMetadata extends RuntimeMetadata {
  const ClassRuntimeMetadata(
      Type type,
      Symbol simpleName,
      Symbol qualifiedName, {
      List annotations: const []})
          : super._(type, annotations, simpleName, qualifiedName);
}

class Query {
  /// If `true`, retains [Metadata.annotations] on fields, methods, and classes.
  final bool includeAnnotations;

  /// If non-empty, only returns fields, methods, or classes with the provided
  /// [Type] annotated on them.
  final List<Type> withAnnotations;

  const Query({
    this.includeAnnotations: true,
    this.withAnnotations: const []});
}
